package net.knarcraft.minecraftserverlauncher.server.servertypes;

import net.knarcraft.minecraftserverlauncher.Main;
import net.knarcraft.minecraftserverlauncher.profile.ServerLauncherController;
import net.knarcraft.minecraftserverlauncher.server.JarBuilder;
import net.knarcraft.minecraftserverlauncher.userinterface.GUI;

import java.io.File;
import java.nio.file.Paths;

import static net.knarcraft.minecraftserverlauncher.utility.CommonFunctions.downloadFile;

/**
 * This class represents the CraftBukkit Minecraft server type
 */
public class CraftBukkit extends AbstractServerType {

    private final String downloadURLPart;

    /**
     * Instantiates a new CraftBukkit server type
     *
     * @param typeName        <p>The name of the server type</p>
     * @param isProxy         <p>Whether this server type is a proxy server</p>
     * @param versions        <p>A list of one or more server versions for the type</p>
     * @param downloadURL     <p>The URL used for downloading .jar files</p>
     * @param downloadURLPart <p>A string used after the download url as an additional part of the URL</p>
     */
    public CraftBukkit(String typeName, boolean isProxy, String[] versions, String downloadURL, String downloadURLPart) {
        super(typeName, isProxy, versions, downloadURL);
        this.downloadURLPart = downloadURLPart;
    }

    @Override
    public boolean downloadJar(String folder, String version) {
        String targetFile = this.getName() + version + ".jar";
        String file = downloadURLPart + version + ".jar";
        File filePath = new File(folder + targetFile);

        if (version.equals("Latest")) {
            return filePath.isFile() || buildJar(folder);
        } else {
            return filePath.isFile() || downloadFile(downloadURL + file, Paths.get(filePath.toURI()));
        }
    }

    /**
     * Builds the necessary .jar file
     *
     * @param folder <p>The folder the build file should be moved to</p>
     * @return <p>True if the build was successful</p>
     */
    private boolean buildJar(String folder) {
        String buildToolsDirectory = Main.getApplicationWorkDirectory() + File.separator + "files" + File.separator +
                "BuildTools" + File.separator;

        GUI gui = ServerLauncherController.getInstance().getGUI();
        JarBuilder jarBuilder = new JarBuilder(buildToolsDirectory, folder, gui);
        if (this instanceof Spigot) {
            return jarBuilder.buildSpigotJar();
        } else {
            return jarBuilder.buildBukkitJar();
        }
    }

}
